@@ -26,6 +26,9 @@
 #include <QResource>
 #include <QSet>
 
+#include <private/qguiapplication_p.h>
+#include <qpa/qplatformtheme.h>
+
 #include <qplatformdefs.h>
 
 #include <array>
@@ -60,13 +63,24 @@ void initRCCIconTheme()
 }
 Q_COREAPP_STARTUP_FUNCTION(initRCCIconTheme)
 
-// Set the icon theme fallback to breeze
-// Most of our apps use "lots" of icons that most of the times
-// are only available with breeze, we still honour the user icon
-// theme but if the icon is not found there, we go to breeze
-// since it's almost sure it'll be there
+// Makes sure the icon theme fallback is set to breeze or one of its
+// variants. Most of our apps use "lots" of icons that most of the times
+// are only available with breeze, we still honour the user icon theme
+// but if the icon is not found there, we go to breeze since it's almost
+// sure it'll be there
 static void setBreezeFallback()
 {
+    if (const QPlatformTheme *theme = QGuiApplicationPrivate::platformTheme()) {
+        const QVariant themeHint = theme->themeHint(QPlatformTheme::SystemIconFallbackThemeName);
+        if (themeHint.isValid()) {
+            const QString iconTheme = themeHint.toString();
+            if (iconTheme.contains(QStringLiteral("breeze"), Qt::CaseInsensitive)) {
+                QIcon::setFallbackThemeName(iconTheme);
+                return;
+            }
+        }
+    }
+
     QIcon::setFallbackThemeName(QStringLiteral("breeze"));
 }
 
@@ -157,11 +171,14 @@ public:
     }
 
 private:
-    bool mbValid;
-    KIconLoader::Type mType;
+    bool mbValid = false;
+    KIconLoader::Type mType = KIconLoader::Fixed;
     KIconLoader::Context mContext;
-    int mSize, mScale, mMinSize, mMaxSize;
-    int mThreshold;
+    int mSize = 0;
+    int mScale = 1;
+    int mMinSize = 1;
+    int mMaxSize = 50;
+    int mThreshold = 2;
 
     const QString mBaseDir;
     const QString mThemeDir;
@@ -261,7 +278,6 @@ KIconTheme::KIconTheme(const QString &name, const QString &appName, const QStrin
     d->mInternalName = name;
 
     QStringList themeDirs;
-    QSet<QString> addedDirs; // Used for avoiding duplicates.
 
     // Applications can have local additions to the global "locolor" and
     // "hicolor" icon themes. For these, the _global_ theme description
@@ -272,9 +288,10 @@ KIconTheme::KIconTheme(const QString &name, const QString &appName, const QStrin
         && (name == defaultThemeName()
             || name == QLatin1String("hicolor")
             || name == QLatin1String("locolor"))) { /* clang-format on */
-        const QStringList dataDirs = QStandardPaths::standardLocations(QStandardPaths::GenericDataLocation);
-        for (const auto &dir : dataDirs) {
-            const QString cDir = dir + QLatin1Char('/') + appName + QLatin1String("/icons/") + name + QLatin1Char('/');
+        const QString suffix = QLatin1Char('/') + appName + QLatin1String("/icons/") + name + QLatin1Char('/');
+        QStringList dataDirs = QStandardPaths::standardLocations(QStandardPaths::GenericDataLocation);
+        for (auto &cDir : dataDirs) {
+            cDir += suffix;
             if (QFileInfo::exists(cDir)) {
                 themeDirs += cDir;
             }
@@ -300,20 +317,27 @@ KIconTheme::KIconTheme(const QString &name, const QString &appName, const QStrin
 
     QString fileName;
     QString mainSection;
-    for (const auto &iconDir : std::as_const(icnlibs)) {
-        const QString cDir = iconDir + QLatin1Char('/') + name + QLatin1Char('/');
-        if (QDir(cDir).exists()) {
-            themeDirs += cDir;
-            if (d->mDir.isEmpty()) {
-                if (QFileInfo::exists(cDir + QStringLiteral("index.theme"))) {
-                    d->mDir = cDir;
-                    fileName = d->mDir + QStringLiteral("index.theme");
-                    mainSection = QStringLiteral("Icon Theme");
-                } else if (QFileInfo::exists(cDir + QStringLiteral("index.desktop"))) {
-                    d->mDir = cDir;
-                    fileName = d->mDir + QStringLiteral("index.desktop");
-                    mainSection = QStringLiteral("KDE Icon Theme");
-                }
+    const QString pathSuffix = QLatin1Char('/') + name + QLatin1Char('/');
+    const QLatin1String indexTheme("index.theme");
+    const QLatin1String indexDesktop("theme.desktop");
+    for (auto &iconDir : icnlibs) {
+        iconDir += pathSuffix;
+        const QFileInfo fi(iconDir);
+        if (!fi.exists() || !fi.isDir()) {
+            continue;
+        }
+        themeDirs.append(iconDir);
+
+        if (d->mDir.isEmpty()) {
+            QString possiblePath;
+            if (possiblePath = iconDir + indexTheme; QFileInfo::exists(possiblePath)) {
+                d->mDir = iconDir;
+                fileName = possiblePath;
+                mainSection = QStringLiteral("Icon Theme");
+            } else if (possiblePath = iconDir + indexDesktop; QFileInfo::exists(possiblePath)) {
+                d->mDir = iconDir;
+                fileName = possiblePath;
+                mainSection = QStringLiteral("KDE Icon Theme");
             }
         }
     }
@@ -347,6 +371,7 @@ KIconTheme::KIconTheme(const QString &name, const QString &appName, const QStrin
     d->mExtensions =
         cfg.readEntry("KDE-Extensions", QStringList{QStringLiteral(".png"), QStringLiteral(".svgz"), QStringLiteral(".svg"), QStringLiteral(".xpm")});
 
+    QSet<QString> addedDirs; // Used for avoiding duplicates.
     const QStringList dirs = cfg.readPathEntry("Directories", QStringList()) + cfg.readPathEntry("ScaledDirectories", QStringList());
     for (const auto &dirName : dirs) {
         KConfigGroup cg(d->sharedConfig, dirName);
@@ -449,27 +474,25 @@ QList<int> KIconTheme::querySizes(KIconLoader::Group group) const
     return d->m_iconGroups[group].availableSizes;
 }
 
+static bool isAnyOrDirContext(const KIconThemeDir *dir, KIconLoader::Context context)
+{
+    return context == KIconLoader::Any || context == dir->context();
+}
+
 QStringList KIconTheme::queryIcons(int size, KIconLoader::Context context) const
 {
     // Try to find exact match
     QStringList result;
-    const auto listDirs = d->mDirs + d->mScaledDirs;
-    for (KIconThemeDir *dir : listDirs) {
-        if ((context != KIconLoader::Any) && (context != dir->context())) {
+    const QVector<KIconThemeDir *> listDirs = d->mDirs + d->mScaledDirs;
+    for (const KIconThemeDir *dir : listDirs) {
+        if (!isAnyOrDirContext(dir, context)) {
             continue;
         }
-        if ((dir->type() == KIconLoader::Fixed) && (dir->size() == size)) {
-            result += dir->iconList();
-            continue;
-        }
-        if (dir->type() == KIconLoader::Scalable //
-            && size >= dir->minSize() //
-            && size <= dir->maxSize()) {
-            result += dir->iconList();
-            continue;
-        }
-        if ((dir->type() == KIconLoader::Threshold) //
-            && (abs(size - dir->size()) < dir->threshold())) {
+
+        const int dirSize = dir->size();
+        if ((dir->type() == KIconLoader::Fixed && dirSize == size) //
+            || (dir->type() == KIconLoader::Scalable && size >= dir->minSize() && size <= dir->maxSize())
+            || (dir->type() == KIconLoader::Threshold && abs(size - dirSize) < dir->threshold())) {
             result += dir->iconList();
         }
     }
@@ -482,7 +505,7 @@ QStringList KIconTheme::queryIconsByContext(int size, KIconLoader::Context conte
     int dw;
 
     // We want all the icons for a given context, but we prefer icons
-    // of size size . Note that this may (will) include duplicate icons
+    // of size "size" . Note that this may (will) include duplicate icons
     // QStringList iconlist[34]; // 33 == 48-16+1
     QStringList iconlist[128]; // 33 == 48-16+1
     // Usually, only the 0, 6 (22-16), 10 (32-22), 16 (48-32 or 32-16),
@@ -490,7 +513,7 @@ QStringList KIconTheme::queryIconsByContext(int size, KIconLoader::Context conte
     // will make icon themes with different icon sizes.
     const auto listDirs = d->mDirs + d->mScaledDirs;
     for (KIconThemeDir *dir : listDirs) {
-        if ((context != KIconLoader::Any) && (context != dir->context())) {
+        if (!isAnyOrDirContext(dir, context)) {
             continue;
         }
         dw = abs(dir->size() - size);
@@ -509,7 +532,7 @@ bool KIconTheme::hasContext(KIconLoader::Context context) const
 {
     const auto listDirs = d->mDirs + d->mScaledDirs;
     for (KIconThemeDir *dir : listDirs) {
-        if ((context == KIconLoader::Any) || (context == dir->context())) {
+        if (isAnyOrDirContext(dir, context)) {
             return true;
         }
     }
@@ -620,12 +643,14 @@ QStringList KIconTheme::list()
             if (theme.startsWith(QLatin1String("default."))) {
                 continue;
             }
-            if (!QFileInfo::exists(iconDir + QLatin1Char('/') + theme + QLatin1String("/index.desktop"))
-                && !QFileInfo::exists(iconDir + QLatin1Char('/') + theme + QLatin1String("/index.theme"))) {
+
+            const QString prefix = iconDir + QLatin1Char('/') + theme;
+            if (!QFileInfo::exists(prefix + QLatin1String("/index.desktop")) //
+                && !QFileInfo::exists(prefix + QLatin1String("/index.theme"))) {
                 continue;
             }
-            KIconTheme oink(theme);
-            if (!oink.isValid()) {
+
+            if (!KIconTheme(theme).isValid()) {
                 continue;
             }
 
@@ -698,13 +723,8 @@ void KIconTheme::assignIconsToContextMenu(ContextMenus type, QList<QAction *> ac
 /*** KIconThemeDir ***/
 
 KIconThemeDir::KIconThemeDir(const QString &basedir, const QString &themedir, const KConfigGroup &config)
-    : mbValid(false)
-    , mType(KIconLoader::Fixed)
-    , mSize(config.readEntry("Size", 0))
+    : mSize(config.readEntry("Size", 0))
     , mScale(config.readEntry("Scale", 1))
-    , mMinSize(1) // just set the variables to something
-    , mMaxSize(50) // meaningful in case someone calls minSize or maxSize
-    , mThreshold(2)
     , mBaseDir(basedir)
     , mThemeDir(themedir)
 {
